@@ -20,10 +20,10 @@
 ******************************************************************************/
 
 /******************************************************************************
-*   file: chat.c - funtions related to p2p chatting                           *
-*           (T)he K(I)cki(N) (T)ickin D(I)kumud Clie(N)t ++ 2.00              *
-*                     coded by Sean Butler 1998                               *
-*                   recoded by Igor van den Hoven 2005                        *
+*               (T)he K(I)cki(N) (T)ickin D(I)kumud Clie(N)t                  *
+*                                                                             *
+*                         coded by Sean Butler 1998                           *
+*                    recoded by Igor van den Hoven 2005                       *
 ******************************************************************************/
 
 #include "tintin.h"
@@ -36,33 +36,23 @@
 #include <arpa/inet.h>
 #include <fcntl.h>
 #include <errno.h>
-#include <pthread.h>
+
+#ifdef HAVE_PTHREAD_H
+	#include <pthread.h>
+#endif
 
 
 #define CALL_TIMEOUT 5
 #define BLOCK_SIZE 500
 #define DEFAULT_PORT 4050
-#define TINTIN_VERSION "TinTin++ 1.96.3"
-
-#define RESTRING(point, value)   \
-{                                \
-	free(point);                \
-	point = strdup((value));    \
-}
-
-#define STRFREE(point)           \
-{                                \
-	free((point));              \
-	point = NULL;               \
-}
 
 DO_COMMAND(do_chat)
 {
-	char left[BUFFER_SIZE], right[BUFFER_SIZE];
+	char *left, *right;
 	int cnt;
 
-	arg = get_arg_in_braces(arg, left, FALSE);
-	arg = get_arg_in_braces(arg, right, TRUE);
+	arg = get_arg_in_braces(arg, &left, FALSE);
+	arg = get_arg_in_braces(arg, &right, TRUE);
 
 	if (*left == 0)
 	{
@@ -70,16 +60,7 @@ DO_COMMAND(do_chat)
 
 		for (cnt = 0 ; *chat_table[cnt].name != 0 ; cnt++)
 		{
-			if (strlen(left) + 20 > ses->cols)
-			{
-				tintin_printf(NULL, "%s", left);
-				*left = 0;
-			}
-			cat_sprintf(left, "%-20s", chat_table[cnt].name);
-		}
-		if (*left)
-		{
-			tintin_printf(NULL, "%s", left);
+			tintin_printf2(ses, "  [%-13s] %s", chat_table[cnt].name, chat_table[cnt].desc);
 		}
 		tintin_header(NULL, "");
 
@@ -93,14 +74,14 @@ DO_COMMAND(do_chat)
 			continue;
 		}
 
-		if (chat_table[cnt].chat != chat_initialize && gtd->chat == NULL)
+		if (chat_table[cnt].fun != chat_initialize && gtd->chat == NULL)
 		{
 			tintin_printf(NULL, "\033[1;31m<CHAT> You must initialize a chat port first.");
 
 			return ses;
 		}
 
-		chat_table[cnt].chat (right);
+		chat_table[cnt].fun(right);
 
 		return ses;
 	}
@@ -121,7 +102,7 @@ DO_CHAT(chat_initialize)
 	struct sockaddr_in sa;
 	struct hostent *hp = NULL;
 	struct linger ld;
-	const char *reuse = "1";
+	char *reuse = "1";
 	int sock;
 
 	if (gtd->chat)
@@ -228,7 +209,8 @@ int chat_new(int s)
 {
 	struct chat_data *new_buddy;
 	struct sockaddr_in sock;
-	int i, fd;
+	unsigned int i;
+	int fd;
 
 	i = sizeof(sock);
 
@@ -274,10 +256,163 @@ int chat_new(int s)
 	Places a call to another chat client.
 */
 
+#ifdef HAVE_GETADDRINFO
+
+void *threaded_chat_call(void *arg)
+{
+	int sock, error;
+	char host[BUFFER_SIZE], port[BUFFER_SIZE], name[BUFFER_SIZE];
+	struct addrinfo *address;
+	static struct addrinfo hints;
+	struct chat_data *new_buddy;
+	struct timeval to;
+	fd_set wds, rds;
+
+	chat_printf("Attempting to call %s ...", arg);
+
+	to.tv_sec = CALL_TIMEOUT;
+	to.tv_usec = 0;
+
+	arg = (void *) cpy_arg_in_braces((char *) arg, host, FALSE);
+	arg = (void *) cpy_arg_in_braces((char *) arg, port, FALSE);
+
+	if (*port == 0)
+	{
+		sprintf(port, "%d", DEFAULT_PORT);
+	}
+
+	hints.ai_family   = AF_UNSPEC;
+	hints.ai_protocol = IPPROTO_TCP;
+	hints.ai_socktype = SOCK_STREAM;
+
+	error = getaddrinfo(host, port, &hints, &address);
+
+	switch (error)
+	{
+		case 0:
+			break;
+
+		case -2:
+			chat_printf("Failed to call %s, unknown host.", host);
+			return NULL;
+
+		default:
+			chat_printf("Failed to call %s.", host);
+			return NULL;
+	}
+
+	if ((sock = socket(address->ai_family, address->ai_socktype, address->ai_protocol)) < 0)
+	{
+		syserr("socket");
+	}
+
+	switch (address->ai_family)
+	{
+		case AF_INET:
+			inet_ntop(address->ai_family, &((struct sockaddr_in *)address->ai_addr)->sin_addr, host, address->ai_addrlen);
+			break;
+
+		case AF_INET6:
+			inet_ntop(address->ai_family, &((struct sockaddr_in6 *)address->ai_addr)->sin6_addr, host, address->ai_addrlen);
+			break;
+	}
+
+	if (connect(sock, address->ai_addr, address->ai_addrlen) != 0)
+	{
+		chat_printf("Failed to connect to %s:%s", host, port);
+
+		close(sock);
+		freeaddrinfo(address);
+
+		return NULL;
+	}
+
+	FD_ZERO(&wds);
+
+	FD_SET(sock, &wds); /* mother chat desc */
+
+	if (select(FD_SETSIZE, NULL, &wds, NULL, &to) == -1)
+	{
+		chat_printf("Failed to connect to %s %s", host, port);
+
+		close(sock);
+		freeaddrinfo(address);
+
+		return NULL;
+	}
+
+	if (!FD_ISSET(sock, &wds))
+	{
+		chat_printf("Connection timed out.");
+
+		close(sock);
+		freeaddrinfo(address);
+
+		return NULL;
+	}
+
+	new_buddy = calloc(1, sizeof(struct chat_data));
+
+	new_buddy->ip    = strdup(host);
+	new_buddy->port  = atoi(port);
+	new_buddy->fd    = sock;
+	new_buddy->name  = strdup("");
+	new_buddy->group = strdup("");
+
+	strip_vt102_codes(gtd->chat->name, name);
+
+	chat_socket_printf(new_buddy, "CHAT:%s\n%s%-5u", name, gtd->chat->ip, gtd->chat->port);
+
+	chat_printf("Socket connected, negotiating protocol...");
+
+	FD_ZERO(&rds);
+	FD_SET(sock, &rds);
+
+	to.tv_sec  = CALL_TIMEOUT;
+	to.tv_usec = 0;
+
+	if (select(FD_SETSIZE, &rds, NULL, NULL, &to) == -1)
+	{
+		close_chat(new_buddy, FALSE);
+		freeaddrinfo(address);
+
+		return NULL;
+	}
+
+	if (process_chat_input(new_buddy) == -1)
+	{
+		FD_CLR(new_buddy->fd, &rds);
+		close_chat(new_buddy, FALSE);
+		freeaddrinfo(address);
+
+		return NULL;
+	}
+
+	if (*new_buddy->name == 0)
+	{
+		close_chat(new_buddy, FALSE);
+	}
+	else
+	{
+		if (fcntl(sock, F_SETFL, O_NDELAY|O_NONBLOCK) == -1)
+		{
+			perror("chat_new: fcntl O_NDELAY|O_NONBLOCK");
+		}
+
+		LINK(new_buddy, gtd->chat->next, gtd->chat->prev);
+		chat_printf("Connection made to %s.", new_buddy->name);
+	}
+	freeaddrinfo(address);
+
+	return NULL;
+}
+
+#else
+
 void *threaded_chat_call(void *arg)
 {
 	int sock, dig;
-	char ip[BUFFER_SIZE], pn[BUFFER_SIZE], name[BUFFER_SIZE];
+	char host[BUFFER_SIZE], port[BUFFER_SIZE], name[BUFFER_SIZE];
 	struct sockaddr_in dest_addr;
 	struct chat_data *new_buddy;
 	struct timeval to;
@@ -288,26 +423,26 @@ void *threaded_chat_call(void *arg)
 	to.tv_sec = CALL_TIMEOUT;
 	to.tv_usec = 0;
 
-	arg = (void *) get_arg_in_braces((const char *) arg, ip, FALSE);
-	arg = (void *) get_arg_in_braces((const char *) arg, pn, FALSE);
+	arg = (void *) cpy_arg_in_braces((char *) arg, host, FALSE);
+	arg = (void *) cpy_arg_in_braces((char *) arg, port, FALSE);
 
-	if (*pn == 0)
+	if (*port == 0)
 	{
-		sprintf(pn, "%d", DEFAULT_PORT);
+		sprintf(port, "%d", DEFAULT_PORT);
 	}
 
-	if (sscanf(ip, "%d.%d.%d.%d", &dig, &dig, &dig, &dig) == 4)
+	if (sscanf(host, "%d.%d.%d.%d", &dig, &dig, &dig, &dig) == 4)
 	{
-		dest_addr.sin_addr.s_addr = inet_addr(ip);
+		dest_addr.sin_addr.s_addr = inet_addr(host);
 	}
 	else
 	{
 		struct hostent *hp;
 		int addr, address[4];
 
-		if ((hp = gethostbyname(ip)) == NULL)
+		if ((hp = gethostbyname(host)) == NULL)
 		{
-			chat_printf("Failed to call %s, unknown host.", ip);
+			chat_printf("Failed to call %s, unknown host.", host);
 
 			return NULL;
 		}
@@ -320,12 +455,12 @@ void *threaded_chat_call(void *arg)
 		address[2] = ( addr >>  8 ) & 0xFF ;
 		address[3] = ( addr       ) & 0xFF ;
 
-		sprintf(ip, "%d.%d.%d.%d", address[0], address[1], address[2], address[3]);
+		sprintf(host, "%d.%d.%d.%d", address[0], address[1], address[2], address[3]);
 	}
 
-	if (is_number(pn))
+	if (is_number(port))
 	{
-		dest_addr.sin_port = htons(atoi(pn));
+		dest_addr.sin_port = htons(atoi(port));
 	}
 	else
 	{
@@ -343,7 +478,7 @@ void *threaded_chat_call(void *arg)
 
 	if (connect(sock, (struct sockaddr *)&dest_addr, sizeof(dest_addr)) != 0)
 	{
-		chat_printf("Failed to connect to %s:%s", ip, pn);
+		chat_printf("Failed to connect to %s:%s", host, port);
 
 		close(sock);
 
@@ -356,7 +491,7 @@ void *threaded_chat_call(void *arg)
 
 	if (select(FD_SETSIZE, NULL, &wds, NULL, &to) == -1)
 	{
-		chat_printf("Failed to connect to %s %s", ip, pn);
+		chat_printf("Failed to connect to %s %s", host, port);
 
 		close(sock);
 
@@ -374,8 +509,8 @@ void *threaded_chat_call(void *arg)
 
 	new_buddy = calloc(1, sizeof(struct chat_data));
 
-	new_buddy->ip    = strdup(ip);
-	new_buddy->port  = atoi(pn);
+	new_buddy->ip    = strdup(host);
+	new_buddy->port  = atoi(port);
 	new_buddy->fd    = sock;
 	new_buddy->name  = strdup("");
 	new_buddy->group = strdup("");
@@ -425,13 +560,29 @@ void *threaded_chat_call(void *arg)
 	return NULL;
 }
 
-void chat_call(const char *arg)
+#endif
+
+#ifdef HAVE_LIBPTHREAD
+
+void chat_call(char *arg)
 {
+	char buf[BUFFER_SIZE];
+
 	pthread_t thread;
 
-	pthread_create(&thread, NULL, (void *) threaded_chat_call, (void *) arg);
+	strcpy(buf, arg);
+
+	pthread_create(&thread, NULL, (void *) threaded_chat_call, (void *) buf);
 }
 	
+#else
+
+void chat_call(char *arg)
+{
+	threaded_chat_call((void *) arg);
+}
+
+#endif
 
 /*
 	Clean up and close a chat conneciton.
@@ -517,9 +668,9 @@ void process_chat_connections(fd_set *read_set, fd_set *write_set, fd_set *exc_s
 	return;
 }
 
-void chat_socket_printf(struct chat_data *buddy, const char *format, ...)
+void chat_socket_printf(struct chat_data *buddy, char *format, ...)
 {
-	char buf[BUFFER_SIZE];
+	char buf[STRING_SIZE];
 	va_list args;
 
 	va_start(args, format);
@@ -537,10 +688,10 @@ void chat_socket_printf(struct chat_data *buddy, const char *format, ...)
 	}
 }
 
-void chat_printf(const char *format, ...)
+void chat_printf(char *format, ...)
 {
 	struct chat_data *buddy;
-	char buf[BUFFER_SIZE], tmp[BUFFER_SIZE];
+	char buf[STRING_SIZE], tmp[STRING_SIZE];
 	va_list args;
 
 	va_start(args, format);
@@ -569,7 +720,7 @@ void chat_printf(const char *format, ...)
 	}
 	sprintf(tmp, "%s%s%s", gtd->chat->color, buf, "\033[0m");
 
-	tintin_puts(tmp, NULL);
+	tintin_puts(NULL, tmp);
 }
 
 /*
@@ -668,7 +819,7 @@ int process_chat_input(struct chat_data *buddy)
 
 			RESTRING(buddy->name, name);
 
-			chat_socket_printf(buddy, "%c%s%c", CHAT_VERSION, TINTIN_VERSION, CHAT_END_OF_COMMAND);
+			chat_socket_printf(buddy, "%c%s%s%c", CHAT_VERSION, "TinTin++ ", VERSION_NUM, CHAT_END_OF_COMMAND);
 
 			sep = strchr(buf, '\n');
 
@@ -710,8 +861,8 @@ void get_chat_commands(struct chat_data *buddy, char *buf, int len)
 
 	push_call("get_chat_commands(%s,%d,%s)",buddy->name,len,buf);
 
-	pti = buf;
-	pto = txt;
+	pti = (unsigned char *) buf;
+	pto = (unsigned char *) txt;
 
 	/*
 		have to deal with these first
@@ -747,7 +898,7 @@ void get_chat_commands(struct chat_data *buddy, char *buf, int len)
 	while (*pti && buddy)
 	{
 		ptc = *pti++;
-		pto = txt;
+		pto = (unsigned char *) txt;
 
 		while (isspace(*pti))
 		{
@@ -824,7 +975,7 @@ void get_chat_commands(struct chat_data *buddy, char *buf, int len)
 			case CHAT_VERSION:
 				if (buddy->version == NULL)
 				{
-					chat_socket_printf(buddy, "%c%s%c", CHAT_VERSION, TINTIN_VERSION, CHAT_END_OF_COMMAND);
+					chat_socket_printf(buddy, "%c%s%s%c", CHAT_VERSION, "TinTin++ ", VERSION_NUM, CHAT_END_OF_COMMAND);
 
 					buddy->version = strdup(txt);
 				}
@@ -1215,12 +1366,12 @@ DO_CHAT(chat_downloaddir)
 DO_CHAT(chat_emote)
 {
 	struct chat_data *buddy;
-	char temp[BUFFER_SIZE], left[BUFFER_SIZE], right[BUFFER_SIZE];
+	char *temp, *left, *right;
 
-	arg = get_arg_in_braces(arg, left,  FALSE);
-	arg = get_arg_in_braces(arg, temp,  TRUE);
+	arg = get_arg_in_braces(arg, &left,  FALSE);
+	arg = get_arg_in_braces(arg, &temp,  TRUE);
 
-	substitute(gtd->ses, temp, right, SUB_COL|SUB_ESC);
+	substitute(gtd->ses, temp, &right, SUB_COL|SUB_ESC);
 
 	if (!strcasecmp(left, "ALL"))
 	{
@@ -1284,12 +1435,12 @@ DO_CHAT(chat_ip)
 DO_CHAT(chat_message)
 {
 	struct chat_data *buddy;
-	char temp[BUFFER_SIZE], left[BUFFER_SIZE], right[BUFFER_SIZE];
+	char *temp, *left, *right;
 
-	arg = get_arg_in_braces(arg, left, FALSE);
-	arg = get_arg_in_braces(arg, temp, TRUE);
+	arg = get_arg_in_braces(arg, &left, FALSE);
+	arg = get_arg_in_braces(arg, &temp, TRUE);
 
-	substitute(gtd->ses, temp, right, SUB_COL|SUB_ESC);
+	substitute(gtd->ses, temp, &right, SUB_COL|SUB_ESC);
 
 	if (!strcasecmp(left, "ALL"))
 	{
@@ -1330,10 +1481,10 @@ DO_CHAT(chat_message)
 
 DO_CHAT(chat_name)
 {
-	char name[BUFFER_SIZE], temp[BUFFER_SIZE];
+	char *name, temp[BUFFER_SIZE];
 	struct chat_data *buddy;
 
-	substitute(gtd->ses, arg, name, SUB_COL|SUB_ESC);
+	substitute(gtd->ses, arg, &name, SUB_COL|SUB_ESC);
 
 	if (!strcmp(gtd->chat->name, name))
 	{
@@ -1361,28 +1512,24 @@ DO_CHAT(chat_name)
 DO_CHAT(chat_paste)
 {
 	struct chat_data *buddy;
-	char temp[BUFFER_SIZE], left[BUFFER_SIZE], right[BUFFER_SIZE];
+	char temp[BUFFER_SIZE], *left, *right;
 
 	if (arg == NULL)
 	{
-		if (strlen(rl_line_buffer))
+		if (strlen(gtd->input_buf))
 		{
-			sprintf(temp, "%s\n%s", gtd->chat->paste_buf, rl_line_buffer);
+			sprintf(temp, "%s\n%s", gtd->chat->paste_buf, gtd->input_buf);
 
 			RESTRING(gtd->chat->paste_buf, temp);
 
-			rl_point = 0;
-
-			rl_delete_text(0, strlen(rl_line_buffer));
-
-			rl_redisplay();
+			cursor_clear_line("");
 		}
 
-		arg = get_arg_in_braces(gtd->chat->paste_buf, left, FALSE);
+		arg = get_arg_in_braces(gtd->chat->paste_buf, &left, FALSE);
 
 		sprintf(temp, "%s\n<078>======================================================================", arg);
 
-		substitute(gtd->ses, temp, right, SUB_COL|SUB_ESC);
+		substitute(gtd->ses, temp, &right, SUB_COL|SUB_ESC);
 
 		RESTRING(gtd->chat->paste_buf, right);
 
@@ -1440,8 +1587,8 @@ DO_CHAT(chat_paste)
 		return;
 	}
 
-	arg = get_arg_in_braces(arg, left, FALSE);
-	arg = get_arg_in_braces(arg, right, TRUE);
+	arg = get_arg_in_braces(arg, &left, FALSE);
+	arg = get_arg_in_braces(arg, &right, TRUE);
 
 	gtd->chat->paste_time = 400000LL + utime();
 
@@ -1465,7 +1612,7 @@ DO_CHAT(chat_peek)
 }
 
 
-void chat_ping(const char *arg)
+void chat_ping(char *arg)
 {
 	struct chat_data *buddy;
 
@@ -1485,11 +1632,11 @@ void chat_ping(const char *arg)
 DO_CHAT(chat_reply)
 {
 	struct chat_data *buddy;
-	char temp[BUFFER_SIZE], left[BUFFER_SIZE];
+	char *temp, *left;
 
-	arg = get_arg_in_braces(arg, temp, TRUE);
+	arg = get_arg_in_braces(arg, &temp, TRUE);
 
-	substitute(gtd->ses, temp, left, SUB_COL|SUB_ESC);
+	substitute(gtd->ses, temp, &left, SUB_COL|SUB_ESC);
 
 	if ((buddy = find_buddy(gtd->chat->reply)) != NULL)
 	{
@@ -1526,12 +1673,12 @@ DO_CHAT(chat_request)
 DO_CHAT(chat_send)
 {
 	struct chat_data *buddy;
-	char temp[BUFFER_SIZE], left[BUFFER_SIZE], right[BUFFER_SIZE];
+	char *temp, *left, *right;
 
-	arg = get_arg_in_braces(arg, left, FALSE);
-	arg = get_arg_in_braces(arg, temp, TRUE);
+	arg = get_arg_in_braces(arg, &left, FALSE);
+	arg = get_arg_in_braces(arg, &temp, TRUE);
 
-	substitute(gtd->ses, temp, right, SUB_COL|SUB_ESC);
+	substitute(gtd->ses, temp, &right, SUB_COL|SUB_ESC);
 
 	if (!strcasecmp(left, "ALL"))
 	{
@@ -1649,10 +1796,10 @@ DO_CHAT(chat_zap)
 
 DO_CHAT(chat_accept)
 {
-	char left[BUFFER_SIZE], right[BUFFER_SIZE];
+	char *left, *right;
 
-	arg = get_arg_in_braces(arg, left, FALSE);
-	arg = get_arg_in_braces(arg, right, TRUE);
+	arg = get_arg_in_braces(arg, &left, FALSE);
+	arg = get_arg_in_braces(arg, &right, TRUE);
 
 	struct chat_data *buddy;
 
@@ -1679,25 +1826,9 @@ DO_CHAT(chat_accept)
 
 	buddy->file_start_time = utime();
 
-	/*
-		I'm sending 3 requests here to speed things up. - Scandum
-	*/
-
-	if (buddy->file_block_tot >= 10 && *right == 'b')
-	{
-		chat_socket_printf(buddy, "%c%c", CHAT_FILE_BLOCK_REQUEST, CHAT_END_OF_COMMAND);
-		chat_socket_printf(buddy, "%c%c", CHAT_FILE_BLOCK_REQUEST, CHAT_END_OF_COMMAND);
-		chat_socket_printf(buddy, "%c%c", CHAT_FILE_BLOCK_REQUEST, CHAT_END_OF_COMMAND);
-		chat_socket_printf(buddy, "%c%c", CHAT_FILE_BLOCK_REQUEST, CHAT_END_OF_COMMAND);
-		chat_socket_printf(buddy, "%c%c", CHAT_FILE_BLOCK_REQUEST, CHAT_END_OF_COMMAND);
-		chat_socket_printf(buddy, "%c%c", CHAT_FILE_BLOCK_REQUEST, CHAT_END_OF_COMMAND);
-		chat_socket_printf(buddy, "%c%c", CHAT_FILE_BLOCK_REQUEST, CHAT_END_OF_COMMAND);
-		chat_socket_printf(buddy, "%c%c", CHAT_FILE_BLOCK_REQUEST, CHAT_END_OF_COMMAND);
-		chat_socket_printf(buddy, "%c%c", CHAT_FILE_BLOCK_REQUEST, CHAT_END_OF_COMMAND);
-	}
 	chat_socket_printf(buddy, "%c%c", CHAT_FILE_BLOCK_REQUEST, CHAT_END_OF_COMMAND);
 
-	chat_printf("Started %sfile transfer from %s, file: %s, size: %lld", !strcasecmp(right, "boost") ? "boosted " : "", buddy->name, buddy->file_name, buddy->file_size);
+	chat_printf("Started file transfer from %s, file: %s, size: %lld", buddy->name, buddy->file_name, buddy->file_size);
 }
 
 
@@ -1738,16 +1869,16 @@ DO_CHAT(chat_decline)
 
 DO_CHAT(chat_sendfile)
 {
-	char left[BUFFER_SIZE];
-	char right[BUFFER_SIZE];
+	char *left;
+	char *right;
 	struct chat_data *buddy;
 
 	/*
 		Determine the chat connection refered to
 	*/
 
-	arg = get_arg_in_braces(arg, left,  0);
-	arg = get_arg_in_braces(arg, right, 0);
+	arg = get_arg_in_braces(arg, &left,  0);
+	arg = get_arg_in_braces(arg, &right, 0);
 
 	if (*left == 0 || *right == 0)
 	{
@@ -2104,11 +2235,11 @@ DO_CHAT(chat_cancelfile)
 
 DO_CHAT(chat_color)
 {
-	char left[BUFFER_SIZE], color[BUFFER_SIZE];
+	char *left, color[BUFFER_SIZE];
 
-	arg = get_arg_in_braces(arg, left, TRUE);
+	arg = get_arg_in_braces(arg, &left, TRUE);
 
-	if (*left == 0 || get_highlight_codes(left, color) == FALSE)
+	if (*left == 0 || get_highlight_codes(gtd->ses, left, color) == FALSE)
 	{
 		chat_printf("Valid colors are:\r\n\r\nreset, bold, faint, underscore, blink, reverse, dim, black, red, green, yellow, blue, magenta, cyan, white, b black, b red, b green, b yellow, b blue, b magenta, b cyan, b white");
 
@@ -2166,11 +2297,11 @@ DO_CHAT(chat_filestat)
 DO_CHAT(chat_group)
 {
 	struct chat_data *buddy;
-	char left[BUFFER_SIZE], right[BUFFER_SIZE];
+	char *left, *right;
 	int cnt = 0;
 
-	arg = get_arg_in_braces(arg, left, FALSE);
-	arg = get_arg_in_braces(arg, right, TRUE);
+	arg = get_arg_in_braces(arg, &left, FALSE);
+	arg = get_arg_in_braces(arg, &right, TRUE);
 
 	if (*left == 0)
 	{
@@ -2327,12 +2458,12 @@ DO_CHAT(chat_ignore)
 DO_CHAT(chat_private)
 {
 	struct chat_data *buddy;
-	char temp[BUFFER_SIZE], left[BUFFER_SIZE], right[BUFFER_SIZE];
+	char *temp, *left, *right;
 
-	arg = get_arg_in_braces(arg, left,  FALSE);
-	arg = get_arg_in_braces(arg, temp,  TRUE);
+	arg = get_arg_in_braces(arg, &left,  FALSE);
+	arg = get_arg_in_braces(arg, &temp,  TRUE);
 
-	substitute(gtd->ses, temp, right, SUB_COL|SUB_ESC);
+	substitute(gtd->ses, temp, &right, SUB_COL|SUB_ESC);
 
 	if (!strcasecmp(left, "ALL"))
 	{
@@ -2371,12 +2502,12 @@ DO_CHAT(chat_private)
 DO_CHAT(chat_public)
 {
 	struct chat_data *buddy;
-	char temp[BUFFER_SIZE], left[BUFFER_SIZE], right[BUFFER_SIZE];
+	char *temp, *left, *right;
 
-	arg = get_arg_in_braces(arg, left,  FALSE);
-	arg = get_arg_in_braces(arg, temp,  TRUE);
+	arg = get_arg_in_braces(arg, &left,  FALSE);
+	arg = get_arg_in_braces(arg, &temp,  TRUE);
 
-	substitute(gtd->ses, temp, right, SUB_COL|SUB_ESC);
+	substitute(gtd->ses, temp, &right, SUB_COL|SUB_ESC);
 
 	if (!strcasecmp(left, "ALL"))
 	{
@@ -2429,7 +2560,7 @@ int get_file_size(char *fpath)
 }
 
 
-struct chat_data *find_buddy(const char *arg)
+struct chat_data *find_buddy(char *arg)
 {
 	struct chat_data *buddy;
 	int cnt = 1;
@@ -2470,7 +2601,7 @@ struct chat_data *find_buddy(const char *arg)
 }
 
 
-struct chat_data *find_group(const char *arg)
+struct chat_data *find_group(char *arg)
 {
 	struct chat_data *buddy;
 
