@@ -19,19 +19,15 @@
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *******************************************************************************/
 
-/*********************************************************************/
-/* file: main.c - main module - signal setup/shutdown etc            */
-/*                             TINTIN++                              */
-/*          (T)he K(I)cki(N) (T)ickin D(I)kumud Clie(N)t             */
-/*                     coded by peter unold 1992                     */
-/*********************************************************************/
-
-/* note: a bunch of changes were made here to add readline support -- daw */
+/******************************************************************************
+*                (T)he K(I)cki(N) (T)ickin D(I)kumud Clie(N)t                 *
+*                                                                             *
+*                         coded by Peter Unold 1992                           *
+******************************************************************************/
 
 #include "tintin.h"
 
 #include <signal.h>
-#include <fcntl.h>
 #include <sys/socket.h>
 
 /*************** globals ******************/
@@ -39,31 +35,23 @@
 struct session *gts;
 struct tintin_data *gtd;
 
-RETSIGTYPE pipe_handler(int no_care)
+
+
+void pipe_handler(int signal)
 {
 	tintin_printf(NULL, "broken_pipe: dumping stack");
 
 	dump_stack();
-
-	if (signal(SIGPIPE, pipe_handler) == BADSIG)
-	{
-		syserr("signal SIGPIPE");
-	}
-
 }
 
 /*
 	when the screen size changes, take note of it
 */
 
-RETSIGTYPE winchhandler(int no_care)
+void winch_handler(int signal)
 {
 	struct session *ses;
 
-	/*
-		select() will see a "syscall interrupted" error; remember not to worry
-	*/
-
 	for (ses = gts ; ses ; ses = ses->next)
 	{
 		init_screen_size(ses);
@@ -75,16 +63,53 @@ RETSIGTYPE winchhandler(int no_care)
 	}
 
 	/*
-		we haveta reinitialize the signals for sysv machines
-	*/
+		we have to reinitialize the signals for sysv machines
 
-	if (signal(SIGWINCH, winchhandler) == BADSIG)
+	if (signal(SIGWINCH, winch_handler) == BADSIG)
 	{
 		syserr("signal SIGWINCH");
 	}
+	*/
 }
 
 
+void abort_handler(int signal)
+{
+	if (gtd->ses->connect_retry > utime())
+	{
+		gtd->ses->connect_retry = 0;
+	}
+	else if (HAS_BIT(gtd->ses->telopts, TELOPT_FLAG_SGA) && !HAS_BIT(gtd->ses->telopts, TELOPT_FLAG_ECHO))
+	{
+		socket_printf(gtd->ses, 1, "%c", 3);
+	}
+	else if (gtd->ses == gts)
+	{
+		quitmsg(NULL);
+	}
+	else
+	{
+		do_zap(gtd->ses, "");
+	}
+}
+
+void suspend_handler(int signal)
+{
+	printf("\033[r\033[%d;%dH", gtd->ses->rows, 1);
+
+	fflush(stdout);
+
+	reset_terminal();
+
+	kill(0, SIGSTOP);
+
+	dirty_screen(gtd->ses);
+
+	init_terminal();
+
+	tintin_puts(NULL, "#RETURNING BACK TO TINTIN++.");
+}
+
 void trap_handler(int signal)
 {
 	static char crashed = FALSE;
@@ -95,6 +120,8 @@ void trap_handler(int signal)
 	}
 	crashed = TRUE;
 
+	reset_terminal();
+
 	clean_screen(gtd->ses);
 
 	dump_stack();
@@ -112,8 +139,6 @@ void trap_handler(int signal)
 
 int main(int argc, char **argv)
 {
-	char filestring[256];
-
 	#ifdef SOCKS
 		SOCKSinit(argv[0]);
 	#endif
@@ -122,7 +147,6 @@ int main(int argc, char **argv)
 
 	init_tintin();
 
-
 	if (signal(SIGTERM, trap_handler) == BADSIG)
 	{
 		syserr("signal SIGTERM");
@@ -138,58 +162,85 @@ int main(int argc, char **argv)
 		syserr("signal SIGHUP");
 	}
 
-	if (signal(SIGABRT, myquitsig) == BADSIG)
+	if (signal(SIGABRT, abort_handler) == BADSIG)
 	{
 		syserr("signal SIGTERM");
 	}
 
-	if (signal(SIGINT, myquitsig) == BADSIG)
+	if (signal(SIGINT, abort_handler) == BADSIG)
 	{
 		syserr("signal SIGINT");
 	}
 
+	if (signal(SIGTSTP, suspend_handler) == BADSIG)
+	{
+		syserr("signal SIGSTOP");
+	}
+
 	if (signal(SIGPIPE, pipe_handler) == BADSIG)
 	{
 		syserr("signal SIGPIPE");
 	}
 
-	if (signal(SIGWINCH, winchhandler) == BADSIG)
+	if (signal(SIGWINCH, winch_handler) == BADSIG)
 	{
 		syserr("signal SIGWINCH");
 	}
 
 	if (getenv("HOME") != NULL)
 	{
-		sprintf(filestring, "%s/%s", getenv("HOME"), HISTORY_FILE);
+		char filename[256];
 
-		read_history(filestring);
+		sprintf(filename, "%s/%s", getenv("HOME"), HISTORY_FILE);
 
-		add_history("");
+		read_history(gts, filename);
 	}
+
 	srand48(time(NULL));
 
 	if (argc > 1)
 	{
-		if (!strcmp(argv[1], "-v"))
-		{
-			do_configure(gtd->ses, "{VERBOSE} {ON}");
+		char c;
 
-			do_read(gtd->ses, argv[2]);
-		}
-		else if (!strcmp(argv[1], "-e"))
+		while ((c = getopt(argc, argv, "e: h r: t: v")) != EOF)
 		{
-			gtd->ses = parse_input(argv[2], gtd->ses);
+			switch (c)
+			{
+				case 'e':
+					gtd->ses = parse_input(gtd->ses, optarg);
+					break;
+
+				case 'h':
+					tintin_printf(NULL, "Usage: %s [-v] [file] [-e \"command\"] [-r file] [-t title]", argv[0]);
+					reset_terminal();
+					exit(1);
+					break;
+
+				case 'r':
+					gtd->ses = do_read(gtd->ses, optarg);
+					break;
+
+				case 't':
+					printf("\033]0;%s\007", optarg);
+					break;
+
+				case 'v':
+					do_configure(gtd->ses, "{VERBOSE} {ON}");
+					break;
+
+				default:
+					tintin_printf(NULL, "Unknown option '%c'.", c);
+					break;
+			}
 		}
-		else if (!strcmp(argv[1], "-h"))
+		
+		if (argv[optind] != NULL)
 		{
-			tintin_printf(NULL, "Usage: %s [file] [-v file] [-e \"command\"]", argv[0]);
+			gtd->ses = do_read(gtd->ses, argv[optind]);
 		}
-		else
-		{
-			gtd->ses = do_read(gtd->ses, argv[1]);
-		}
-	}
-	commandloop();
+	} 
+
+	mainloop();
 
 	pop_call();
 	return 0;
@@ -202,21 +253,23 @@ void init_tintin(void)
 
 	gts                 = calloc(1, sizeof(struct session));
 
-	for (cnt = 0 ; cnt < LIST_ALL ; cnt++)
+	for (cnt = 0 ; cnt < LIST_MAX ; cnt++)
 	{
-		gts->list[cnt] = init_list();
+		gts->list[cnt] = init_list(cnt);
 	}
 
 	gts->name           = strdup("gts");
+	gts->class          = strdup("");
 	gts->host           = strdup("");
 	gts->port           = strdup("");
 	gts->telopts        = TELOPT_FLAG_ECHO;
-	gts->map_size       = 6;
-	gts->scroll_line    = -1;
 
 	gtd                 = calloc(1, sizeof(struct tintin_data));
 
 	gtd->ses            = gts;
+
+	gtd->mem            = calloc(1, sizeof(struct memory_data));
+
 	gtd->str_hash_size  = sizeof(struct str_hash_data);
 
 	gtd->mccp_buf_max   = 64;
@@ -237,8 +290,6 @@ void init_tintin(void)
 		}
 	}
 
-	initrl();
-
 	init_screen_size(gts);
 
 	/*
@@ -247,8 +298,6 @@ void init_tintin(void)
 
 	printf("\033=\033[?1h");
 
-	SET_BIT(gts->flags, SES_FLAG_VERBOSE);
-
 	do_configure(gts, "{SPEEDWALK}         {OFF}");
 	do_configure(gts, "{VERBATIM}          {OFF}");
 	do_configure(gts, "{REPEAT ENTER}      {OFF}");
@@ -258,21 +307,81 @@ void init_tintin(void)
 	do_configure(gts, "{LOG}               {RAW}");
 	do_configure(gts, "{BUFFER SIZE}      {5000}");
 	do_configure(gts, "{SCROLL LOCK}        {ON}");
-	do_configure(gts, "{HISTORY SIZE}     {1000}");
+	do_configure(gts, "{HISTORY SIZE}     {2500}");
 	do_configure(gts, "{CONNECT RETRY}      {60}");
 	do_configure(gts, "{PACKET PATCH}        {0}");
 	do_configure(gts, "{TINTIN CHAR}         {#}");
-	do_configure(gts, "{VERBATIM CHAR}    {\\\\}");
+	do_configure(gts, "{VERBATIM CHAR}      {\\}");
 	do_configure(gts, "{REPEAT CHAR}         {!}");
 
-	DEL_BIT(gts->flags, SES_FLAG_VERBOSE);
+	insertnode_list(gts,  "n",  "s",  "1", LIST_PATHDIR);
+	insertnode_list(gts,  "e",  "w",  "2", LIST_PATHDIR);
+	insertnode_list(gts,  "s",  "n",  "4", LIST_PATHDIR);
+	insertnode_list(gts,  "w",  "e",  "8", LIST_PATHDIR);
+	insertnode_list(gts,  "u",  "d", "16", LIST_PATHDIR);
+	insertnode_list(gts,  "d",  "u", "32", LIST_PATHDIR);
 
-	insertnode_list(gts, "n", "s", "0", LIST_PATHDIR);
-	insertnode_list(gts, "e", "w", "0", LIST_PATHDIR);
-	insertnode_list(gts, "s", "n", "0", LIST_PATHDIR);
-	insertnode_list(gts, "w", "e", "0", LIST_PATHDIR);
-	insertnode_list(gts, "u", "d", "0", LIST_PATHDIR);
-	insertnode_list(gts, "d", "u", "0", LIST_PATHDIR);
+	insertnode_list(gts, "ne", "sw",  "3", LIST_PATHDIR);
+	insertnode_list(gts, "nw", "se",  "9", LIST_PATHDIR);
+	insertnode_list(gts, "se", "nw",  "6", LIST_PATHDIR);
+	insertnode_list(gts, "sw", "ne", "12", LIST_PATHDIR);
 
-	do_help(gtd->ses, "CREDITS");
+	init_terminal();
+
+	do_showme(gts, "");
+	do_showme(gts, "<068>      #<068>###################################################################<068>#");
+	do_showme(gts, "<068>      #<078>                                                                   <068>#");
+	do_showme(gts, "<068>      #<078>                           T I N T I N + +                         <068>#");
+	do_showme(gts, "<068>      #<078>                                                                   <068>#");
+	do_showme(gts, "<068>      #<078>            (<068>T<078>)he k(<068>I<078>)cki(<068>N<078>) (<068>T<078>)ickin d(<068>I<078>)kumud clie(<068>N<078>)t           <068>#");
+	do_showme(gts, "<068>      #<078>                                                                   <068>#");
+	do_showme(gts, "<068>      #<078>                 Original TINTIN code by Peter Unold               <068>#");
+	do_showme(gts, "<068>      #<078>       new code by Bill Reiss, David A. Wagner, Rob Ellsworth,     <068>#");
+	do_showme(gts, "<068>      #<078>                 Jeremy C. Jack, Igor van den Hoven                <068>#");
+	do_showme(gts, "<068>      #<078>                             1992, 2007                            <068>#");
+	do_showme(gts, "<068>      #<078>                                                                   <068>#");
+	do_showme(gts, "<068>      #<078>                               "VERSION_NUM"                              <068>#");
+	do_showme(gts, "<068>      #<078>                                                                   <068>#");
+	do_showme(gts, "<068>      #<068>###################################################################<068>#<088>");
+	do_showme(gts, "");
+}
+
+
+void quitmsg(char *message)
+{
+	struct session *ses;
+
+	while ((ses = gts->next) != NULL)
+	{
+		cleanup_session(ses);
+	}
+
+	if (gtd->chat)
+	{
+		close(gtd->chat->fd);
+	}
+
+	if (gtd->history_size)
+	{
+		char filename[BUFFER_SIZE];
+
+		sprintf(filename, "%s/%s", getenv("HOME"), HISTORY_FILE);
+
+		write_history(gts, filename);
+	}
+
+	reset_terminal();
+
+	clean_screen(gts);
+
+	if (message)
+	{
+		printf("\n%s\n", message);
+	}
+
+	printf("\nGoodbye from TinTin++\n\n");
+
+	fflush(NULL);
+
+	exit(0);
 }
