@@ -20,10 +20,10 @@
 ******************************************************************************/
 
 /******************************************************************************
-*   file: math.c - funtions related to mathematical expressions               *
-*           (T)he K(I)cki(N) (T)ickin D(I)kumud Clie(N)t ++ 2.00              *
-*                         coded by Bill Reiss 1993                            *
-*                   recoded by Igor van den Hoven 2004                        *
+*                (T)he K(I)cki(N) (T)ickin D(I)kumud Clie(N)t                 *
+*                                                                             *
+*                          coded by Bill Reiss 1993                           *
+*                     recoded by Igor van den Hoven 2004                      *
 ******************************************************************************/
 
 #include "tintin.h"
@@ -54,23 +54,21 @@ int precision;
 
 DO_COMMAND(do_math)
 {
-	char left[BUFFER_SIZE], right[BUFFER_SIZE], result[BUFFER_SIZE];
+	char *left, *right;
 	struct listroot *root;
 
 	root = ses->list[LIST_VARIABLE];
 
-	arg = get_arg_in_braces(arg, left,  FALSE);
-	arg = get_arg_in_braces(arg, right, TRUE);
+	arg = get_arg_in_braces(arg, &left,  FALSE);
+	arg = get_arg_in_braces(arg, &right, TRUE);
 
 	if (*left == 0 || *right == 0)
 	{
-		tintin_printf(ses, "#SYNTAX: #MATH {variable} {expression}");
+		tintin_printf2(ses, "#SYNTAX: #MATH {variable} {expression}");
 	}
 	else
 	{
-		sprintf(result, "{%s} {%.*f}", left, precision, mathexp(ses, right));
-
-		do_variable(ses, result);
+		internal_variable(ses, "{%s} {%.*f}", left, precision, mathexp(ses, right));
 	}
 	return ses;
 }
@@ -78,58 +76,55 @@ DO_COMMAND(do_math)
 
 DO_COMMAND(do_if)
 {
-	char left[BUFFER_SIZE], true[BUFFER_SIZE], false[BUFFER_SIZE], temp[BUFFER_SIZE];
+	char *left, *true, *false, *temp;
 
-	arg = get_arg_in_braces(arg, left,  0);
-	arg = get_arg_in_braces(arg, true,  1);
-	arg = get_arg_in_braces(arg, false, 1);
+	arg = get_arg_in_braces(arg, &left,  0);
+	arg = get_arg_in_braces(arg, &true,  1);
+	arg = get_arg_in_braces(arg, &false, 1);
 
 	if (*left == 0 || *true == 0)
 	{
-		tintin_printf(ses, "#SYNTAX: #IF {expression} {true} {false}");
+		tintin_printf2(ses, "#SYNTAX: #IF {expression} {true} {false}");
 	}
 	else
 	{
-		substitute(ses, left, temp, SUB_VAR|SUB_FUN);
+		substitute(ses, left, &temp, SUB_VAR|SUB_FUN);
 
 		if (mathexp(ses, temp))
 		{
-			ses = parse_input(true, ses);
+			ses = parse_input(ses, true);
 		}
 		else if (*false)
 		{
-			ses = parse_input(false, ses);
+			ses = parse_input(ses, false);
 		}
 	}
 	return ses;
 }
 
-double get_number(struct session *ses, const char *str)
+double get_number(struct session *ses, char *str)
 {
-	char number[BUFFER_SIZE];
+	char *number;
+
+	substitute(ses, str, &number, SUB_VAR|SUB_FUN);
 
-	substitute(ses, str, number, SUB_VAR|SUB_FUN);
-	
 	return mathexp(ses, number);
 }
 
-char *get_number_string(struct session *ses, const char *str)
+void get_number_string(struct session *ses, char *str, char **result)
 {
-	char number[BUFFER_SIZE];
-	static char result[BUFFER_SIZE];
+	char *number;
 
-	substitute(ses, str, number, SUB_VAR|SUB_FUN);	
+	substitute(ses, str, &number, SUB_VAR|SUB_FUN);
 
-	sprintf(result, "%.*f", precision, mathexp(ses, number));
-
-	return result;
+	*result = stringf_alloc("%.*f", precision, mathexp(ses, number));
 }
 
-/******************************************************************************
-* mathematical expression interpreter by Scandum                              *
-******************************************************************************/
+/*
+	Flexible tokenized mathematical expression interpreter - Igor
+*/
 
-double mathexp(struct session *ses, const char *str)
+double mathexp(struct session *ses, char *str)
 {
 	struct listnode *node;
 
@@ -137,7 +132,7 @@ double mathexp(struct session *ses, const char *str)
 
 	if (mathexp_tokenize(ses, str) == FALSE)
 	{
-		tintin_printf(ses, "#MATH EXP: INVALID INPUT {%s}", str);
+		tintin_printf2(ses, "#MATH EXP: INVALID INPUT {%s}", str);
 
 		return 0;
 	}
@@ -176,9 +171,9 @@ double mathexp(struct session *ses, const char *str)
 	point = -1;                                                  \
 }
 
-int mathexp_tokenize(struct session *ses, const char *str)
+int mathexp_tokenize(struct session *ses, char *str)
 {
-	char buf1[BUFFER_SIZE], buf2[BUFFER_SIZE], buf3[BUFFER_SIZE], *pti, *pta;
+	char buf1[BUFFER_SIZE], buf2[BUFFER_SIZE], buf3[STRING_SIZE], *pti, *pta;
 	int level, status, valid, point;
 
 	level     = 0;
@@ -187,8 +182,8 @@ int mathexp_tokenize(struct session *ses, const char *str)
 	valid     = FALSE;
 	precision = 0;
 
-	pta = (char *) buf3;
-	pti = (char *) str;
+	pta = buf3;
+	pti = str;
 
 	kill_list(ses, LIST_MATH);
 
@@ -240,7 +235,7 @@ int mathexp_tokenize(struct session *ses, const char *str)
 					case '"':
 						if (pta != buf3)
 						{
-							tintin_printf2(NULL, "MATH EXP: \" FOUND INSIDE INTEGER");
+							tintin_printf2(NULL, "MATH EXP: \" FOUND INSIDE A NUMBER");
 							return FALSE;
 						}
 						*pta++ = *pti++;
@@ -250,7 +245,7 @@ int mathexp_tokenize(struct session *ses, const char *str)
 					case '(':
 						if (pta != buf3)
 						{
-							tintin_printf2(NULL, "MATH EXP: PARANTESES FOUND INSIDE INTEGER");
+							tintin_printf2(NULL, "#MATH EXP: PARANTESES FOUND INSIDE A NUMBER");
 							return FALSE;
 						}
 						*pta++ = *pti++;
@@ -264,6 +259,11 @@ int mathexp_tokenize(struct session *ses, const char *str)
 
 					case '.':
 						*pta++ = *pti++;
+						if (point >= 0)
+						{
+							tintin_printf2(NULL, "#MATH EXP: MORE THAN ONE POINT FOUND INSIDE A NUMBER");
+							return FALSE;
+						}
 						point++;
 						break;
 
@@ -515,7 +515,7 @@ void mathexp_compute(struct listnode *node)
 		case 'd':
 			if (tintoi(node->next->pr) <= 0)
 			{
-				tintin_printf(NULL, "#MATHEXP: INVALID DICE: %s", node->next->pr);
+				tintin_printf2(NULL, "#MATHEXP: INVALID DICE: %s", node->next->pr);
 				value = 0;
 			}
 			else
@@ -529,7 +529,7 @@ void mathexp_compute(struct listnode *node)
 		case '/':
 			if (tintoi(node->next->pr) == 0)
 			{
-				tintin_printf(NULL, "#MATH ERROR: DIVISION ZERO.");
+				tintin_printf2(NULL, "#MATH ERROR: DIVISION ZERO.");
 				value = tintoi(node->prev->pr);
 			}
 			else
@@ -547,7 +547,7 @@ void mathexp_compute(struct listnode *node)
 		case '%':
 			if (tintoi(node->next->pr) == 0)
 			{
-				tintin_printf(NULL, "#MATH ERROR: MODULO ZERO.");
+				tintin_printf2(NULL, "#MATH ERROR: MODULO ZERO.");
 				value = tintoi(node->prev->pr);
 			}
 			else
@@ -658,7 +658,7 @@ void mathexp_compute(struct listnode *node)
 	node->pr = strdup(temp);
 }
 
-double tintoi(const char *str)
+double tintoi(char *str)
 {
 	switch (str[0])
 	{
@@ -679,7 +679,7 @@ double tintoi(const char *str)
 	}
 }
 
-double tincmp(const char *left, const char *right)
+double tincmp(char *left, char *right)
 {
 	switch (left[0])
 	{
@@ -691,19 +691,19 @@ double tincmp(const char *left, const char *right)
 	}
 }
 
-double tineval(const char *left, const char *right)
+double tineval(char *left, char *right)
 {
 	switch (left[0])
 	{
 		case '"':
-			return regexp(left, right);
+			return regexp(left, right, TRUE);
 
 		default:
 			return tintoi(left) == tintoi(right);
 	}
 }
 			
-double tindice(const char *left, const char *right)
+double tindice(char *left, char *right)
 {
 	long long cnt, numdice, sizedice, sum;
 
