@@ -19,14 +19,11 @@
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *******************************************************************************/
 
-/*********************************************************************/
-/* file: parse.c - some utility-functions                            */
-/*                             TINTIN III                            */
-/*          (T)he K(I)cki(N) (T)ickin D(I)kumud Clie(N)t             */
-/*                     coded by peter unold 1992                     */
-/*********************************************************************/
-
-/* note: a bunch of changes were made here for readline support -- daw */
+/******************************************************************************
+*                (T)he K(I)cki(N) (T)ickin D(I)kumud Clie(N)t                 *
+*                                                                             *
+*                         coded by Peter Unold 1992                           *
+******************************************************************************/
 
 #include "tintin.h"
 
@@ -34,11 +31,11 @@
 /* parse input, check for TINTIN commands and aliases and send to session */
 /**************************************************************************/
 
-struct session *parse_input(const char *input, struct session *ses)
+struct session *parse_input(struct session *ses, char *input)
 {
-	char command[BUFFER_SIZE], arg[BUFFER_SIZE];
+	char *command, *arg1, *arg2, *unwind;
 
-	if (push_call("[%s] parse_input(%s,%p)",ses->name,input,ses))
+	if (push_call("[%s] parse_input(%p,%s)",ses->name,ses,input))
 	{
 		pop_call();
 		return ses;
@@ -56,70 +53,96 @@ struct session *parse_input(const char *input, struct session *ses)
 
 	if (HAS_BIT(ses->flags, SES_FLAG_VERBATIM) && *input != gtd->tintin_char)
 	{
-		sprintf(command, "%s\r\n", input);
+		substitute(ses, input, &command, SUB_EOL);
 
 		write_line_mud(command, ses);
 
+		string_free(command);
+
 		pop_call();
 		return(ses);
 	}
 
 	if (*input == gtd->verbatim_char)
 	{
-		sprintf(command, "%s\r\n", &input[1]);
+		substitute(ses, &input[1], &command, SUB_EOL);
 
 		write_line_mud(command, ses);
 
+		string_free(command);
+
 		pop_call();
 		return ses;
 	}
 
 	while (*input)
 	{
-		if (*input == ';')
+		unwind = string_alloc("");
+
+		if (*input == COMMAND_SEPARATOR)
 		{
 			if (*++input == 0)
 			{
 				break;
 			}
 		}
-		input = get_arg_stop_spaces(input, arg);
+		input = get_arg_stop_spaces(input, &command);
 
-		substitute(ses, arg, command, SUB_VAR|SUB_FUN);
+		substitute(ses, command, &arg1, SUB_VAR|SUB_FUN);
 
-		input = get_arg_all(input, arg);
-
-		if (*command == gtd->tintin_char)
+		if (strcmp(arg1, command))
 		{
-			ses = parse_tintin_command(command + 1, arg, ses);
+			command = stringf_alloc("%s%s", arg1, input);
+
+			ses = parse_input(ses, command);
+
+			pop_call();
+			return ses;
+		}
+
+		input = get_arg_all(input, &arg2);
+
+		if (*arg1 == gtd->tintin_char)
+		{
+			ses = parse_tintin_command(ses, arg1 + 1, arg2);
 		}
 		else
 		{
-			if (*arg)
+			if (check_all_aliases(ses, arg1, arg2, &command))
 			{
-				cat_sprintf(command, " %s", arg);
-			}
+				DEL_BIT(gtd->flags, TINTIN_FLAG_USERCOMMAND); /* get rid of spam for aliasses */
 
-			if (check_all_aliases(command, arg, ses))
-			{
-				DEL_BIT(gts->flags, SES_FLAG_USERCOMMAND);
-
-				ses = parse_input(command, ses);
-			}
-			else if (HAS_BIT(ses->flags, SES_FLAG_SPEEDWALK) && !*arg && is_speedwalk_dirs(command))
-			{
-				process_speedwalk(command, ses);
+				ses = parse_input(ses, command);
 			}
 			else
 			{
-				write_mud(ses, command);
+				if (*arg2)
+				{
+					command = stringf_alloc("%s %s", arg1, arg2);
+				}
+				else
+				{
+					command = string_alloc(arg1);
+				}
+			
+				if (HAS_BIT(ses->flags, SES_FLAG_SPEEDWALK) && *arg2 == 0 && is_speedwalk_dirs(command))
+				{
+					process_speedwalk(command, ses);
+				}
+				else
+				{
+					write_mud(ses, command);
+				}
 			}
 		}
+
+		string_free(unwind);
+
 		if (HAS_BIT(ses->flags, SES_FLAG_BREAK))
 		{
 			break;
 		}
-  	}
+	}
 
 	pop_call();
 	return ses;
@@ -129,7 +152,7 @@ struct session *parse_input(const char *input, struct session *ses)
 /* return TRUE if commands only consists of capital letters N,S,E ... */
 /**********************************************************************/
 
-int is_speedwalk_dirs(const char *cp)
+int is_speedwalk_dirs(char *cp)
 {
 	int flag = FALSE;
 
@@ -171,7 +194,7 @@ int is_speedwalk_dirs(const char *cp)
 /* do the speedwalk thing */
 /**************************/
 
-void process_speedwalk(const char *cp, struct session *ses)
+void process_speedwalk(char *cp, struct session *ses)
 {
 	char sc[2];
 	int loopcnt, i;
@@ -209,9 +232,9 @@ void process_speedwalk(const char *cp, struct session *ses)
 /* parse most of the tintin-commands */
 /*************************************/
 
-struct session *parse_tintin_command(const char *command, char *arg, struct session *ses)
+struct session *parse_tintin_command(struct session *ses, char *command, char *arg)
 {
-	char newcommand[BUFFER_SIZE], argument[BUFFER_SIZE];
+	char *temp;
 	int cmd;
 	struct session *sesptr;
 
@@ -221,9 +244,12 @@ struct session *parse_tintin_command(const char *command, char *arg, struct sess
 		{
 			if (*arg)
 			{
-				get_arg_in_braces(arg, arg, TRUE);
-				substitute(ses, arg, newcommand, SUB_VAR|SUB_FUN);
-				parse_input(newcommand, sesptr);  /* was: #sessioname commands */
+				get_arg_in_braces(arg, &temp, TRUE);
+
+				substitute(ses, temp, &command, SUB_VAR|SUB_FUN);
+
+				parse_input(sesptr, command);
+
 				return ses;
 			}
 			else
@@ -237,11 +263,11 @@ struct session *parse_tintin_command(const char *command, char *arg, struct sess
 	{
 		int i = atoi(command);
 
-		get_arg_in_braces(arg, arg, TRUE);
+		get_arg_in_braces(arg, &command, TRUE);
 
 		while (i-- > 0)
 		{
-			ses = parse_input(arg, ses);
+			ses = parse_input(ses, command);
 		}
 		return ses;
 	}
@@ -256,14 +282,9 @@ struct session *parse_tintin_command(const char *command, char *arg, struct sess
 			{
 				if (HAS_BIT(command_table[cmd].flags, CMD_FLAG_SUB))
 				{
-					substitute(ses, arg, argument, SUB_VAR|SUB_FUN);
-
-					return (ses = (*command_table[cmd].command) (ses, argument));
-				}
-				else
-				{
-					return (ses = (*command_table[cmd].command) (ses, arg));
+					substitute(ses, arg, &arg, SUB_VAR|SUB_FUN);
 				}
+				return (ses = (*command_table[cmd].command) (ses, arg));
 			}
 		}
 	}
@@ -274,84 +295,90 @@ struct session *parse_tintin_command(const char *command, char *arg, struct sess
 
 
 /*
-	get all arguments - only check for unescaped ';'
+	get all arguments - only check for unescaped command separators
 */
 
-const char *get_arg_all(const char *s, char *arg)
+char *get_arg_all(char *string, char **result)
 {
+	char *pto, *pti, buffer[STRING_SIZE];
 	int nest = 0;
 
-	s = space_out(s);
+	pti = space_out(string);
+	pto = buffer;
 
-	while (*s)
+	while (*pti)
 	{
 #ifdef BIG5
-		if (*s & 0x80)
+		if (*pti & 0x80)
 		{
-			*arg++ = *s++;
-			if (*s)
+			*pto++ = *pti++;
+
+			if (*pti)
 			{
-				*arg++ = *s++;
+				*pto++ = *pti++;
 			}
 			continue;
 		}
 #endif
-
-		if (*s == '\\' && s[1] == ';')
+		if (*pti == '\\' && pti[1] == COMMAND_SEPARATOR)
 		{
-			*arg++ = *s++;
+			*pto++ = *pti++;
 		}
-		else if (*s == ';' && nest == 0)
+		else if (*pti == COMMAND_SEPARATOR && nest == 0)
 		{
 			break;
 		}
-		else if (*s == DEFAULT_OPEN)
+		else if (*pti == DEFAULT_OPEN)
 		{
 			nest++;
 		}
-		else if (*s == DEFAULT_CLOSE)
+		else if (*pti == DEFAULT_CLOSE)
 		{
 			nest--;
 		}
-		*arg++ = *s++;
+		*pto++ = *pti++;
 	}
-	*arg = '\0';
-	return s;
+	*pto = '\0';
+
+	*result = string_alloc(buffer);
+
+	return pti;
 }
 
 /*
 	Braces are stripped in braced arguments leaving all else as is.
 */
 
-const char *get_arg_in_braces(const char *s, char *arg, int flag)
+char *get_arg_in_braces(char *string, char **result, int flag)
 {
+	char *pti, *pto, buffer[STRING_SIZE];
 	int nest = 1;
-	const char *ptr;
 
-	s = space_out(s);
-	ptr = s;
+	pti = space_out(string);
+	pto = buffer;
 
-	if (*s != DEFAULT_OPEN)
+	if (*pti != DEFAULT_OPEN)
 	{
 		if (flag == FALSE)
 		{
-			s = get_arg_stop_spaces(ptr, arg);
+			pti = get_arg_stop_spaces(pti, result);
 		}
 		else
 		{
-			s = get_arg_with_spaces(ptr, arg);
+			pti = get_arg_with_spaces(pti, result);
 		}
-		return s;
+		return pti;
 	}
-	s++;
 
-	while (*s)
+	pti++;
+
+	while (*pti)
 	{
 #ifdef BIG5
-		if (*s & 0x80)
+		if (*pti & 0x80)
 		{
-			*arg++ = *s++;
-			if (*s == 0)
+			*pto++ = *pti++;
+			if (*pti == 0)
 			{
 				break;
 			}
@@ -359,11 +386,11 @@ const char *get_arg_in_braces(const char *s, char *arg, int flag)
 		}
 #endif
 
-		if (*s == DEFAULT_OPEN)
+		if (*pti == DEFAULT_OPEN)
 		{
 			nest++;
 		}
-		else if (*s == DEFAULT_CLOSE)
+		else if (*pti == DEFAULT_CLOSE)
 		{
 			nest--;
 
@@ -372,167 +399,329 @@ const char *get_arg_in_braces(const char *s, char *arg, int flag)
 				break;
 			}
 		}
-		*arg++ = *s++;
+		*pto++ = *pti++;
 	}
 
-	if (*s == 0)
+	if (*pti == 0)
 	{
 		tintin_printf2(NULL, "#Unmatched braces error!");
 	}
 	else
 	{
-		s++;
+		pti++;
 	}
-	*arg = '\0';
+	*pto = '\0';
 
-	return s;
+	*result = string_alloc(buffer);
+
+	return pti;
 }
 
-/**************************************/
-/* get all arguments                  */
-/**************************************/
 
-const char *get_arg_with_spaces(const char *s, char *arg)
+/*
+	get all arguments
+*/
+
+char *get_arg_with_spaces(char *string, char **result)
 {
+	char *pto, *pti, buffer[STRING_SIZE];
 	int nest = 0;
 
-	while (*s)
+	pti = space_out(string);
+	pto = buffer;
+
+	while (*pti)
 	{
 #ifdef BIG5
-		if (*s & 0x80)
+		if (*pti & 0x80)
 		{
-			*arg++ = *s++;
-			if (*s == 0)
+			*pto++ = *pti++;
+			if (*pti == 0)
 			{
 				break;
 			}
+			continue;
 		}
-		continue;
 #endif
 
-		if (*s == '\\')
+		if (*pti == '\\' && pti[1] == COMMAND_SEPARATOR)
 		{
-			*arg++ = *s++;
-
-			if (*s)
-			{
-				*arg++ = *s++;
-			}
+			*pto++ = *pti++;
 		}
-		else if (*s == ';' && nest == 0)
+		else if (*pti == COMMAND_SEPARATOR && nest == 0)
 		{
 			break;
 		}
-		else if (*s == DEFAULT_OPEN)
+		else if (*pti == DEFAULT_OPEN)
 		{
-			*arg++ = *s++;
 			nest++;
 		}
-		else if (*s == DEFAULT_CLOSE)
+		else if (*pti == DEFAULT_CLOSE)
 		{
-			*arg++ = *s++;
 			nest--;
 		}
-		else
-		{
-			*arg++ = *s++;
-		}
+		*pto++ = *pti++;
 	}
-	*arg = '\0'; 
-	return s;
+	*pto = '\0'; 
+
+	*result = string_alloc(buffer);
+
+	return pti;
 }
 
 /*
 	get one arg, stop at spaces
 */
 
-const char *get_arg_stop_spaces(const char *s, char *arg)
+char *get_arg_stop_spaces(char *string, char **result)
 {
+	char *pto, *pti, buffer[STRING_SIZE];
 	int nest = 0;
 
-	while (*s)
+	pti = space_out(string);
+	pto = buffer;
+
+	while (*pti)
 	{
 #ifdef BIG5
-		if(*s & 0x80)
+		if(*pti & 0x80)
 		{
-			*arg++ = *s++;
-			if (*s)
+			*pto++ = *pti++;
+			if (*pti)
 			{
-				*arg++ = *s++;
+				*pto++ = *pti++;
 			}
 			continue;
 		}
 #endif
 
-		if (*s == '\\')
+		if (*pti == '\\' && pti[1] == COMMAND_SEPARATOR)
 		{
-			*arg++ = *s++;
-
-			if (*s)
-			{
-				*arg++ = *s++;
-			}
+			*pto++ = *pti++;
 		}
-		else if (*s == ';' && nest == 0)
+		else if (*pti == COMMAND_SEPARATOR && nest == 0)
 		{
 			break;
 		}
-		else if (*s == ' ' && nest == 0)
+		else if (*pti == ' ' && nest == 0)
 		{
 			break;
 		}
-		else if (*s == DEFAULT_OPEN)
+		else if (*pti == DEFAULT_OPEN)
 		{
-			*arg++ = *s++;
 			nest++;
 		}
-		else if (*s == DEFAULT_CLOSE)
+		else if (*pti == DEFAULT_CLOSE)
 		{
-			*arg++ = *s++;
 			nest--;
 		}
-		else
-		{
-			*arg++ = *s++;
-		}
+		*pto++ = *pti++;
 	}
-	*arg = '\0';
+	*pto = '\0';
 
-	return s;
+	*result = string_alloc(buffer);
+
+	return pti;
 }
 
 /*
-	spaceout - advance ptr to next none-space
+	get_arg_in_braces for threaded and old school routine.
 */
 
-const char *space_out(const char *s)
+char *cpy_arg_in_braces(char *pti, char *pto, int flag)
 {
-	while (isspace(*s))
+	int nest = 1;
+
+	pti = space_out(pti);
+
+	if (*pti != DEFAULT_OPEN)
 	{
-		s++;
+		if (flag == FALSE)
+		{
+			pti = cpy_arg_stop_spaces(pti, pto);
+		}
+		else
+		{
+			pti = cpy_arg_with_spaces(pti, pto);
+		}
+		return pti;
 	}
-	return(s);
+
+	pti++;
+
+	while (*pti)
+	{
+#ifdef BIG5
+		if (*pti & 0x80)
+		{
+			*pto++ = *pti++;
+			if (*pti == 0)
+			{
+				break;
+			}
+			continue;
+		}
+#endif
+
+		if (*pti == DEFAULT_OPEN)
+		{
+			nest++;
+		}
+		else if (*pti == DEFAULT_CLOSE)
+		{
+			nest--;
+
+			if (nest == 0)
+			{
+				break;
+			}
+		}
+		*pto++ = *pti++;
+	}
+
+	if (*pti == 0)
+	{
+		tintin_printf2(NULL, "#Unmatched braces error!");
+	}
+	else
+	{
+		pti++;
+	}
+	*pto = '\0';
+
+	return pti;
+}
+
+
+/*
+	get all arguments
+*/
+
+char *cpy_arg_with_spaces(char *pti, char *pto)
+{
+	int nest = 0;
+
+	pti = space_out(pti);
+
+	while (*pti)
+	{
+#ifdef BIG5
+		if (*pti & 0x80)
+		{
+			*pto++ = *pti++;
+			if (*pti == 0)
+			{
+				break;
+			}
+			continue;
+		}
+#endif
+
+		if (*pti == '\\' && pti[1] == COMMAND_SEPARATOR)
+		{
+			*pto++ = *pti++;
+		}
+		else if (*pti == COMMAND_SEPARATOR && nest == 0)
+		{
+			break;
+		}
+		else if (*pti == DEFAULT_OPEN)
+		{
+			nest++;
+		}
+		else if (*pti == DEFAULT_CLOSE)
+		{
+			nest--;
+		}
+		*pto++ = *pti++;
+	}
+	*pto = '\0'; 
+
+	return pti;
+}
+
+/*
+	get one arg, stop at spaces
+*/
+
+char *cpy_arg_stop_spaces(char *pti, char *pto)
+{
+	int nest = 0;
+
+	pti = space_out(pti);
+
+	while (*pti)
+	{
+#ifdef BIG5
+		if(*pti & 0x80)
+		{
+			*pto++ = *pti++;
+			if (*pti)
+			{
+				*pto++ = *pti++;
+			}
+			continue;
+		}
+#endif
+
+		if (*pti == '\\' && pti[1] == COMMAND_SEPARATOR)
+		{
+			*pto++ = *pti++;
+		}
+		else if (*pti == COMMAND_SEPARATOR && nest == 0)
+		{
+			break;
+		}
+		else if (*pti == ' ' && nest == 0)
+		{
+			break;
+		}
+		else if (*pti == DEFAULT_OPEN)
+		{
+			nest++;
+		}
+		else if (*pti == DEFAULT_CLOSE)
+		{
+			nest--;
+		}
+		*pto++ = *pti++;
+	}
+	*pto = '\0';
+
+	return pti;
+}
+
+/*
+	advance ptr to next none-space
+*/
+
+char *space_out(char *string)
+{
+	while (isspace(*string))
+	{
+		string++;
+	}
+	return string;
 }
 
 /*
 	send command+argument to the mud
 */
 
-void write_mud(struct session *ses, const char *command)
+void write_mud(struct session *ses, char *command)
 {
-	char output[BUFFER_SIZE];
+	char *output;
 
-	if (ses->in_room)
+	if (ses->map && ses->map->in_room)
 	{
 		follow_map(ses, command);
 	}
-
 	if (HAS_BIT(ses->flags, SES_FLAG_MAPPING))
 	{
 		check_insert_path(command, ses);
 	}
 
-	substitute(ses, command, output, SUB_VAR|SUB_FUN|SUB_ESC|SUB_EOL);
+	substitute(ses, command, &output, SUB_VAR|SUB_FUN|SUB_ESC|SUB_EOL);
 
 	write_line_mud(output, ses);
 }
@@ -543,35 +732,40 @@ void write_mud(struct session *ses, const char *command)
 * substituted beforehand                                                      *
 ******************************************************************************/
 
-void do_one_line(char *line, struct session *ses)
+void do_one_line(char **line, struct session *ses)
 {
-	char strip[BUFFER_SIZE];
+	char strip[STRING_SIZE];
 
-	push_call("[%s] do_one_line(%p,%p)",ses->name,line,ses);
+	push_call("[%s] do_one_line(%p,%p)",ses->name,*line,ses);
 
-	strip_vt102_codes(line, strip);
+	strip_vt102_codes(*line, strip);
 
 	if (!HAS_BIT(ses->list[LIST_ACTION]->flags, LIST_FLAG_IGNORE))
 	{
-		check_all_actions(line, strip, ses);
+		check_all_actions(ses, *line, strip);
 	}
 
 	if (!HAS_BIT(ses->list[LIST_PROMPT]->flags, LIST_FLAG_IGNORE))
 	{
 		if (HAS_BIT(ses->flags, SES_FLAG_SPLIT))
 		{
-			check_all_prompts(line, strip, ses);
+			check_all_prompts(ses, *line, strip);
 		}
 	}
 
+	if (!HAS_BIT(ses->list[LIST_GAG]->flags, LIST_FLAG_IGNORE))
+	{
+		check_all_gags(ses, *line, strip);
+	}
+
 	if (!HAS_BIT(ses->list[LIST_SUBSTITUTE]->flags, LIST_FLAG_IGNORE))
 	{
-		check_all_substitutions(line, strip, ses);
+		check_all_substitutions(ses, line, strip);
 	}
 
 	if (!HAS_BIT(ses->list[LIST_HIGHLIGHT]->flags, LIST_FLAG_IGNORE))
 	{
-		check_all_highlights(line, strip, ses);
+		check_all_highlights(ses, line, strip);
 	}
 
 	pop_call();
