@@ -17,39 +17,38 @@
 *   You should have received a copy of the GNU General Public License         *
 *   along with this program; if not, write to the Free Software               *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
-*******************************************************************************/
+******************************************************************************/
 
-/*********************************************************************/
-/* file: split.c - functions related to the prompt command           */
-/*                             TINTIN III                            */
-/*          (T)he K(I)cki(N) (T)ickin D(I)kumud Clie(N)t             */
-/*                     coded by Igor van den Hoven 2004              */
-/*********************************************************************/
+/******************************************************************************
+*                (T)he K(I)cki(N) (T)ickin D(I)kumud Clie(N)t                 *
+*                                                                             *
+*                      coded by Igor van den Hoven 2004                       *
+******************************************************************************/
 
 #include "tintin.h"
 
 
 DO_COMMAND(do_prompt)
 {
-	char left[BUFFER_SIZE], right[BUFFER_SIZE], pr[BUFFER_SIZE];
+	char *left, *right, *line;
 	struct listroot *root;
 
 	root = ses->list[LIST_PROMPT];
 
-	arg = get_arg_in_braces(arg, left,  FALSE);
-	arg = get_arg_in_braces(arg, right, TRUE);
-	arg = get_arg_in_braces(arg, pr,    TRUE);
+	arg = get_arg_in_braces(arg, &left,  FALSE);
+	arg = get_arg_in_braces(arg, &right, TRUE);
+	arg = get_arg_in_braces(arg, &line,    TRUE);
 
-	if (!*pr)
+	if (*line == 0)
 	{
-		sprintf(pr, "%d", 1);
+		line = string_alloc("1");
 	}
 
 	if (*left == 0)
 	{
 		show_list(ses, root, LIST_PROMPT);
 	}
-	else if (*left && !*right)
+	else if (*left && *right == 0)
 	{
 		if (show_node_with_wild(ses, left, LIST_PROMPT) == FALSE)
 		{
@@ -58,9 +57,9 @@ DO_COMMAND(do_prompt)
 	}
 	else
 	{
-		updatenode_list(ses, left, right, pr, LIST_PROMPT);
+		updatenode_list(ses, left, right, line, LIST_PROMPT);
 
-		show_message(ses, LIST_PROMPT, "#OK. {%s} NOW PROMPTS {%s}.", left, right);
+		show_message(ses, LIST_PROMPT, "#OK. {%s} NOW PROMPTS {%s} @ {%s}.", left, right, line);
 	}
 	return ses;
 }
@@ -68,32 +67,13 @@ DO_COMMAND(do_prompt)
 
 DO_COMMAND(do_unprompt)
 {
-	char left[BUFFER_SIZE];
-	struct listroot *root;
-	struct listnode *node;
-	int found = FALSE;
+	delete_node_with_wild(ses, LIST_PROMPT, arg);
 
-	root = ses->list[LIST_PROMPT];
-
-	arg = get_arg_in_braces(arg, left, 1);
-
-	while ((node = search_node_with_wild(root, left)))
-	{
-		show_message(ses, LIST_PROMPT, "#OK. {%s} IS NO LONGER A PROMPT.", node->left);
-
-		deletenode_list(ses, node, LIST_PROMPT);
-
-		found = TRUE;
-	}
-	if (found == FALSE)
-	{
-		show_message(ses, LIST_PROMPT, "#UNPROMPT: NO MATCH(ES) FOUND FOR.", ses);
-	}
 	return ses;
 }
 
 
-void check_all_prompts(char *original, char *line, struct session *ses)
+void check_all_prompts(struct session *ses, char *original, char *line)
 {
 	struct listnode *node;
 
@@ -103,9 +83,11 @@ void check_all_prompts(char *original, char *line, struct session *ses)
 		{
 			if (*node->right)
 			{
-				substitute(ses, node->right, original, SUB_ARG|SUB_VAR|SUB_FUN|SUB_COL);
+				substitute(ses, node->right, &original, SUB_ARG|SUB_VAR|SUB_FUN|SUB_COL);
 			}
 
+			show_debug(ses, LIST_PROMPT, "#PROMPT DEBUG: %s", original);
+
 			do_one_prompt(ses, original, atoi(node->pr));
 
 			SET_BIT(ses->flags, SES_FLAG_GAG);
@@ -133,17 +115,15 @@ void do_one_prompt(struct session *ses, char *prompt, int row)
 
 	if (row <= ses->top_row && row >= ses->bot_row)
 	{
-		if (HAS_BIT(ses->list[LIST_PROMPT]->flags, LIST_FLAG_DEBUG))
-		{
-			tintin_printf2(ses, "#PROMPT ERROR, INVALID ROW: {%s}", prompt);
-		}
+		show_message(ses, LIST_PROMPT, "#ERROR: INVALID PROMPT ROW: {%s} {%d}", prompt, row);
+
 		return;
 	}
 	strip_vt102_codes(prompt, temp);
 
 	if (strlen(temp) == 0)
 	{
-		sprintf(temp, "%.*s", ses->cols + 4, "\033[0m----------------------------------------------------------------------------------------------------------------------------------------------------------------");
+		sprintf(temp, "%.*s", ses->cols + 4, "\033[0m----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------");
 	}
 	else if ((int) strlen(temp) <= ses->cols)
 	{
