@@ -20,11 +20,10 @@
 ******************************************************************************/
 
 /******************************************************************************
-*   file: telopt.c - funtions related to telnet negotations                   *
-*           (T)he K(I)cki(N) (T)ickin D(I)kumud Clie(N)t ++ 2.00              *
-*                     coded by peter unold 1992                               *
-*                     new code by Igor van den Hoven 2004                     *
-*******************************************************************************/
+*                (T)he K(I)cki(N) (T)ickin D(I)kumud Clie(N)t                 *
+*                                                                             *
+*                      coded by Igor van den Hoven 2004                       *
+******************************************************************************/
 
 #include "tintin.h"
 #include "telnet.h"
@@ -65,15 +64,7 @@ struct iac_type
 	int  (* func) (struct session *ses, int cplen, unsigned char *cpsrc);
 };
 
-
-/*
-	despite the fact the first three are written, they should not run in an
-	ideal world, after we have session initialization code. - jdoiel
-
-	some muds use ttype - Scandum
-*/
-
-const struct iac_type iac_table [] =
+struct iac_type iac_table [] =
 {
 	{   3,  iac_do_sga,           &send_will_sga           },
 	{   3,  iac_do_ttype,         &send_will_ttype         },
@@ -104,7 +95,7 @@ const struct iac_type iac_table [] =
 };
 
 
-void telopt_debug(struct session *ses, const char *format, ...)
+void telopt_debug(struct session *ses, char *format, ...)
 {
 	char buf[BUFFER_SIZE];
 	va_list args;
@@ -115,7 +106,7 @@ void telopt_debug(struct session *ses, const char *format, ...)
 		vsprintf(buf, format, args);
 		va_end(args);
 
-		tintin_puts2(buf, ses);
+		tintin_puts2(ses, buf);
 	}
 }
 
@@ -159,7 +150,8 @@ void translate_telopts(struct session *ses, unsigned char *src, int cplen)
 				break;
 
 			case Z_STREAM_END:
-				tintin_puts2("\n\r#COMPRESSION END, DISABLING MCCP.", ses);
+				tintin_puts2(ses, "");
+				tintin_puts2(ses, "#COMPRESSION END, DISABLING MCCP.");
 				cplen = ses->mccp->next_out - gtd->mccp_buf;
 				cpsrc = gtd->mccp_buf;
 				inflateEnd(ses->mccp);
@@ -168,7 +160,8 @@ void translate_telopts(struct session *ses, unsigned char *src, int cplen)
 				break;
 
 			default:
-				tintin_puts2("\n\r#COMPRESSION ERROR, RESETTING MCCP.", ses);
+				tintin_puts2(ses, "");
+				tintin_puts2(ses, "#COMPRESSION ERROR, RESETTING MCCP.");
 				send_dont_mccp2(ses, 0, NULL);
 				send_do_mccp2(ses, 0, NULL);
 				inflateEnd(ses->mccp);
@@ -230,9 +223,9 @@ void translate_telopts(struct session *ses, unsigned char *src, int cplen)
 
 					case GA:
 					case EOR:
-						sprintf(cpdst, "RCVD %s", TELCMD(cpsrc[os]));
-						gtd->mud_output_len += strlen(cpdst);
-						cpdst += strlen(cpdst);
+						sprintf((char *) cpdst, "RCVD %s", TELCMD(cpsrc[os])); 
+						gtd->mud_output_len += strlen((char *) cpdst);
+						cpdst += strlen((char *) cpdst);
 						break;
 
 					case DO:
@@ -322,7 +315,7 @@ void translate_telopts(struct session *ses, unsigned char *src, int cplen)
 						}
 						else
 						{
-							tintin_puts("#IAC BAD TELOPT", NULL);
+							tintin_puts(NULL, "#IAC BAD TELOPT");
 							skip = 1;
 						}
 						break;
@@ -432,8 +425,6 @@ int send_will_sga(struct session *ses, int cplen, unsigned char *cpsrc)
 
 		telopt_debug(ses, "SENT IAC WILL SGA");
 	}
-	check_character_mode(ses);
-
 	return 3;
 }
 
@@ -447,8 +438,6 @@ int send_wont_sga(struct session *ses, int cplen, unsigned char *cpsrc)
 
 		telopt_debug(ses, "SENT IAC WONT SGA");
 	}
-	check_character_mode(ses);
-
 	return 3;
 }
 
@@ -482,8 +471,6 @@ int send_do_sga(struct session *ses, int cplen, unsigned char *cpsrc)
 	}
 	SET_BIT(ses->telopts, TELOPT_FLAG_SGA);
 
-	check_character_mode(ses);
-
 	return 3;
 }
 
@@ -566,14 +553,20 @@ int send_wont_new_environ(struct session *ses, int cplen, unsigned char *cpsrc)
 
 int send_sb_naws(struct session *ses, int cplen, unsigned char *cpsrc)
 {
-	if (atoi(ses->port) != TELNET_PORT)
+	int rows;
+
+	if (!HAS_BIT(ses->telopts, TELOPT_FLAG_INIT_NAWS))
 	{
+		SET_BIT(ses->telopts, TELOPT_FLAG_INIT_NAWS);
+
 		socket_printf(ses, 3, "%c%c%c", IAC, WILL, TELOPT_NAWS);
 
 		telopt_debug(ses, "SENT IAC WILL NAWS");
 	}
 
-	socket_printf(ses, 9, "%c%c%c%c%c%c%c%c%c", IAC, SB, TELOPT_NAWS, 0, ses->cols % 255, 0, ses->rows % 255, IAC, SE);
+	rows = HAS_BIT(ses->flags, SES_FLAG_SPLIT) ? ses->bot_row - ses->top_row + 1 : ses->rows;
+
+	socket_printf(ses, 9, "%c%c%c%c%c%c%c%c%c", IAC, SB, TELOPT_NAWS, 0, ses->cols % 255, 0, rows % 255, IAC, SE);
 
 	SET_BIT(ses->telopts, TELOPT_FLAG_NAWS);
 
@@ -612,8 +605,10 @@ int send_sb_ttype(struct session *ses, int cplen, unsigned char *cpsrc)
 	}
 	else
 	{
-		socket_printf(ses, 14, "%c%c%c%c%s%c%c", IAC, SB, TELOPT_TTYPE, 0, "TINTIN++", IAC, SE);
-
+		socket_printf(ses, 15, "%c%c%c%c%s%c%c", IAC, SB, TELOPT_TTYPE, 0, "TINTIN++", IAC, SE);
+/*
+		socket_printf(ses, 21, "%c%c%c%c%s %s%c%c", IAC, SB, TELOPT_TTYPE, 0, "TinTin++", VERSION_NUM, IAC, SE);
+*/
 		telopt_debug(ses, "SENT IAC SB TTYPE %s", "TINTIN++");
 	}
 
@@ -652,7 +647,7 @@ int send_echo_on(struct session *ses, int cplen, unsigned char *cpsrc)
 {
 	socket_printf(ses, 3, "%c%c%c", IAC, DONT, TELOPT_ECHO);
 
-	echo_on(ses);
+	SET_BIT(ses->telopts, TELOPT_FLAG_ECHO);
 
 	telopt_debug(ses, "SENT IAC DONT ECHO");
 
@@ -663,7 +658,7 @@ int send_echo_off(struct session *ses, int cplen, unsigned char *cpsrc)
 {
 	socket_printf(ses, 3, "%c%c%c", IAC, DO, TELOPT_ECHO);
 
-	echo_off(ses);
+	DEL_BIT(ses->telopts, TELOPT_FLAG_ECHO);
 
 	telopt_debug(ses, "SENT IAC DO ECHO");
 
@@ -674,7 +669,7 @@ int send_echo_will(struct session *ses, int cplen, unsigned char *cpsrc)
 {
 	socket_printf(ses, 3, "%c%c%c", IAC, WILL, TELOPT_ECHO);
 
-	echo_off(ses);
+	DEL_BIT(ses->telopts, TELOPT_FLAG_ECHO);
 
 	telopt_debug(ses, "SENT IAC WILL ECHO");
 
@@ -782,7 +777,7 @@ int init_mccp(struct session *ses, int cplen, unsigned char *cpsrc)
 
 	if (inflateInit(ses->mccp) != Z_OK)
 	{
-		tintin_puts2("#FAILED TO INITIALIZE MCCP2.", ses);
+		tintin_puts2(ses, "#FAILED TO INITIALIZE MCCP2.");
 		send_dont_mccp2(ses, 0, NULL);
 		free(ses->mccp);
 		ses->mccp = NULL;
