@@ -20,17 +20,13 @@
 *******************************************************************************/
 
 /******************************************************************************
-*   file: vt102.c - funtions related to terminal emulation                    *
-*           (T)he K(I)cki(N) (T)ickin D(I)kumud Clie(N)t ++ 2.00              *
-*                   recoded by Igor van den Hoven 2004                        *
+*                (T)he K(I)cki(N) (T)ickin D(I)kumud Clie(N)t                 *
+*                                                                             *
+*                      coded by Igor van den Hoven 2004                       *
 ******************************************************************************/
 
 #include "tintin.h"
 
-/*
-	don't forget to do a 'fflush(stdout);' after each of these commands!
-*/
-
 void save_pos(struct session *ses)
 {
 	printf("\0337"); 
@@ -104,7 +100,7 @@ void reset_scroll_region(struct session *ses)
 }
 
 
-int skip_vt102_codes(const char *str)
+int skip_vt102_codes(char *str)
 {
 	int skip;
 
@@ -181,7 +177,7 @@ int skip_vt102_codes(const char *str)
 }
 
 
-int skip_vt102_codes_non_graph(const char *str)
+int skip_vt102_codes_non_graph(char *str)
 {
 	int skip = 0;
 
@@ -272,7 +268,7 @@ int skip_vt102_codes_non_graph(const char *str)
 }
 
 
-void strip_vt102_codes(const char *str, char *buf)
+void strip_vt102_codes(char *str, char *buf)
 {
 	char *pti, *pto;
 
@@ -294,12 +290,13 @@ void strip_vt102_codes(const char *str, char *buf)
 	*pto = 0;
 }
 
-void strip_vt102_codes_non_graph(const char *str, char *buf)
+
+void strip_vt102_codes_non_graph(char *str, char *buf)
 {
 	char *pti, *pto;
 
-	pti = (char *) str;
-	pto = (char *) buf;
+	pti = str;
+	pto = buf;
 
 	while (*pti)
 	{
@@ -316,13 +313,13 @@ void strip_vt102_codes_non_graph(const char *str, char *buf)
 	*pto = 0;
 }
 
-void strip_non_vt102_codes(const char *str, char *buf)
+void strip_non_vt102_codes(char *str, char *buf)
 {
 	char *pti, *pto;
 	int len;
 
-	pti = (char *) str;
-	pto = (char *) buf;
+	pti = str;
+	pto = buf;
 
 	while (*pti)
 	{
@@ -341,9 +338,9 @@ void strip_non_vt102_codes(const char *str, char *buf)
 	*pto = 0;
 }
 
-int strip_vt102_strlen(const char *str)
+int strip_vt102_strlen(char *str)
 {
-	const char *pti;
+	char *pti;
 	int i = 0;
 
 	pti = str;
@@ -366,7 +363,7 @@ int strip_vt102_strlen(const char *str)
 }
 
 
-int interpret_vt102_codes(struct session *ses, const char *str, int real)
+int interpret_vt102_codes(struct session *ses, char *str, int real)
 {
 	char data[BUFFER_SIZE] = { 0 };
 	int skip = 0;
@@ -439,38 +436,42 @@ int interpret_vt102_codes(struct session *ses, const char *str, int real)
 				{
 					socket_printf(ses, 5, "%c%c%c%c%c", ESCAPE, '[', '?', '6', 'c');
 				}
+				ses->cur_row = 1;
+				ses->cur_col = 1;
+				ses->sav_row = ses->cur_row;
+				ses->sav_col = ses->cur_col;
 				return FALSE;					
 
 			case 'A':
-				ses->cur_row -= atoi(data);
+				ses->cur_row -= UMAX(1, atoi(data));
 				break;
 
 			case 'B':
 			case 'e':
-				ses->cur_row += atoi(data);
+				ses->cur_row += UMAX(1, atoi(data));
 				break;
 
 			case 'C':
 			case 'a':
-				ses->cur_col += atoi(data);
+				ses->cur_col += UMAX(1, atoi(data));
 				break;
 
 			case 'D':
-				ses->cur_col -= atoi(data);
+				ses->cur_col -= UMAX(1, atoi(data));
 				break;
 
 			case 'E':
-				ses->cur_row -= atoi(data);
+				ses->cur_row -= UMAX(1, atoi(data));
 				ses->cur_col = 1;
 				break;
 
 			case 'F':
-				ses->cur_row -= atoi(data);
+				ses->cur_row -= UMAX(1, atoi(data));
 				ses->cur_col = 1;
 				break;
 
 			case 'G':
-				ses->cur_col = atoi(data);
+				ses->cur_col = UMAX(1, atoi(data));
 				break;
 
 			case 'H':
@@ -502,6 +503,8 @@ int interpret_vt102_codes(struct session *ses, const char *str, int real)
 						ses->bot_row = ses->rows;
 					}
 				}
+				ses->cur_row = 1;
+				ses->cur_col = 1;
 				break;
 
 			case 's':
@@ -531,6 +534,7 @@ int interpret_vt102_codes(struct session *ses, const char *str, int real)
 
 			ses->bot_row = URANGE(1, ses->bot_row, ses->rows);
 
+			
 			return TRUE;
 		}
 	}
